package teen

import (
	"fmt"
	"server/game"
	"server/game/room"
	"server/msg"
	"time"

	"github.com/name5566/leaf/log"
)

// 创建定时管理器
var timerMgr = NewTimerManager()

// 开始发牌，先给庄家发牌
func startDealCard(r *room.Room) {
	initCardDeck(r)
	shuffleCardDeck(r.CardDeck)
	if r.Status == room.RoomStatusWaiting {
		r.Status = room.RoomStatusDealing
		dealer := r.GetDealerPlayer()
		r.RoundSitPos = int(dealer.SitPos)

		players := r.GetPlayersBySitPos(dealer.SitPos)

		// 让所有玩家进行下底
		for _, player := range players {
			delay := time.Duration(300 * time.Millisecond)
			game.Module.Skeleton.AfterFunc(delay, func() {
				chaal(r, player.SitPos, 100)
			})
		}

		time.Sleep(time.Second * 2)

		//每个玩家从第一张开始发，发完再给每个玩家发第二张牌，发完再给每个玩家发第三张牌，要有序有间隔的体现
		for index := 0; index < 3; index++ {
			for i, player := range players {
				player.HandCards = getThreeCards(r)
				player.WaitCard = false
				// 计算延时：轮次*总延时 + 玩家索引*单次延时
				delay := time.Duration(index*len(players)+i) * 300 * time.Millisecond
				game.Module.Skeleton.AfterFunc(delay, func() {
					notifyPlayerDealCard(r, player.SitPos, int32(index))
				})
			}
		}

		game.Module.Skeleton.AfterFunc(time.Second*3, func() {
			notifyPlayerAction(r, dealer.SitPos, msg.PlayerOptType_OPT_SELECT)
			r.Status = room.RoomStatusPlaying
		})

	}
}

// 返回牌堆里3张牌,并从牌堆中移除
func getThreeCards(room *room.Room) *[]msg.ReqCard {
	// 获取前三张牌
	threeCards := (*room.CardDeck)[:3]
	// 移除这三张牌（保留剩余的牌）
	*room.CardDeck = (*room.CardDeck)[3:]
	// 返回这三张牌
	return &threeCards
}

// 通知玩家发牌
func notifyPlayerDealCard(room *room.Room, sitPos int32, index int32) {
	//遍历所有玩家
	for _, player := range room.Players {
		if player.Agent != nil {
			player.Agent.WriteMsg(&msg.ReqDealCard{SitPos: &sitPos, Index: &index})
		}
	}
}

// 通知玩家操作
func notifyPlayerAction(room *room.Room, sitPos int32, optType msg.PlayerOptType) {
	player := room.GetPlayerBySitPos(sitPos)
	if !player.IsRobot {

	} else {
		//人机直接操作
		robotOpt(room, sitPos, optType)
	}
	log.Debug("通知玩家开始操作: %d", sitPos)

	players := room.GetPlayersBySitPos(sitPos)
	for _, p := range players {
		if p.Agent != nil {
			p.Agent.WriteMsg(&msg.ReqPlayerAction{SitPos: int32(sitPos), PlayerOpt: &msg.PlayerOpt{OptType: optType}})
		}
		if sitPos == p.SitPos && !p.IsRobot {
			addPlayerOptTimeout(room, player, time.Second*time.Duration(GameConfig.ThinkTime))
		}
	}
	// game.Module.Skeleton.AfterFunc(time.Second*time.Duration(GameConfig.ThinkTime), func() {

	// })
}

// 添加玩家操作超时事件
func addPlayerOptTimeout(room *room.Room, player *room.Player, timeout time.Duration) {
	eventID := fmt.Sprintf("player_%s_opt_%s", player.Id, player.SitPos)
	timerMgr.AddEvent(eventID, timeout, func() {
		// 超时处理逻辑，弃牌
		packed(room, player.SitPos)
	})
}

// 收到玩家操作
func recvPlayerOptAction(room *room.Room, sitPos int32, opt *msg.PlayerOpt) {
	player := room.GetPlayerBySitPos(sitPos)
	optType := opt.OptType
	log.Debug("recvPlayerOptAction, optType:%v", optType)
	player.LastOpt = &msg.PlayerOpt{OptType: optType}
	if optType == msg.PlayerOptType_OPT_SEEN { // 看牌
		seen(room, sitPos)
	} else {
		cancelOptTimeout(player, sitPos)
		if optType == msg.PlayerOptType_OPT_PACKED { // 弃牌
			packed(room, sitPos)
		} else if optType == msg.PlayerOptType_OPT_CHAAL { // 跟注
			chaal(room, sitPos, opt.BetAmount)
			game.Module.Skeleton.AfterFunc(time.Second*1, func() {
				sendRoomBet(room)
			})
		} else if optType == msg.PlayerOptType_OPT_SHOW { // 亮牌
			show(room, sitPos)
		}
	}
	updateAllPlayerInfo(room)
}

// 发送消息给所有玩家
func SendRoundMsgToAll(room *room.Room, mg *msg.ReqRound) {
	if mg.PlayerOpt.OptType == msg.PlayerOptType_OPT_SEEN {
		if mg.RoundSitPos != SelfSitPos {
			mg.PlayerOpt.SeenCards = nil
		}
	}
	for _, player := range room.Players {
		if player.Agent != nil {
			player.Agent.WriteMsg(mg)
		}
	}
	room.AddPlayerOpt(mg)
}

// 更新所有玩家的操作
func updateAllPlayerInfo(room *room.Room) {
	for _, player := range room.Players {
		if player.Agent != nil {
			player.Agent.WriteMsg(&msg.ReqPlayerList{ReqPlayers: convertToMsgPlayerList(room.Players)})
		}
	}
}

// 发送房间押注
func sendRoomBet(room *room.Room) {
	for _, player := range room.Players {
		if player.Agent != nil {
			player.Agent.WriteMsg(&msg.ReqBet{BetAmount: &room.GameRound.TotalBet})
		}
	}
}
